@@ -8,6 +8,7 @@ from typing import Any
 from typing import Dict
 
 from sqlalchemy import Column
+from sqlalchemy import create_mock_engine
 from sqlalchemy import inspect
 from sqlalchemy import MetaData
 from sqlalchemy import String
@@ -17,6 +18,7 @@ from sqlalchemy import text
 from sqlalchemy.testing import config
 from sqlalchemy.testing import mock
 from sqlalchemy.testing.assertions import eq_
+from sqlalchemy.testing.fixtures import FutureEngineMixin
 from sqlalchemy.testing.fixtures import TablesTest as SQLAlchemyTablesTest
 from sqlalchemy.testing.fixtures import TestBase as SQLAlchemyTestBase
 
@@ -26,9 +28,7 @@ from ..environment import EnvironmentContext
 from ..migration import MigrationContext
 from ..operations import Operations
 from ..util import sqla_compat
-from ..util.sqla_compat import create_mock_engine
-from ..util.sqla_compat import sqla_14
-from ..util.sqla_compat import sqla_1x
+from ..util.sqla_compat import sqla_2
 
 
 testing_config = configparser.ConfigParser()
@@ -36,10 +36,7 @@ testing_config.read(["test.cfg"])
 
 
 class TestBase(SQLAlchemyTestBase):
-    if sqla_1x:
-        is_sqlalchemy_future = False
-    else:
-        is_sqlalchemy_future = True
+    is_sqlalchemy_future = sqla_2
 
     @testing.fixture()
     def ops_context(self, migration_context):
@@ -52,6 +49,12 @@ class TestBase(SQLAlchemyTestBase):
             connection, opts=dict(transaction_per_migration=True)
         )
 
+    @testing.fixture
+    def as_sql_migration_context(self, connection):
+        return MigrationContext.configure(
+            connection, opts=dict(transaction_per_migration=True, as_sql=True)
+        )
+
     @testing.fixture
     def connection(self):
         with config.db.connect() as conn:
@@ -62,14 +65,6 @@ class TablesTest(TestBase, SQLAlchemyTablesTest):
     pass
 
 
-if sqla_14:
-    from sqlalchemy.testing.fixtures import FutureEngineMixin
-else:
-
-    class FutureEngineMixin:  # type:ignore[no-redef]
-        __requires__ = ("sqlalchemy_14",)
-
-
 FutureEngineMixin.is_sqlalchemy_future = True
 
 
@@ -137,7 +132,6 @@ def op_fixture(
     literal_binds=False,
     native_boolean=None,
 ):
-
     opts = {}
     if naming_convention:
         opts["target_metadata"] = MetaData(naming_convention=naming_convention)
@@ -188,12 +182,8 @@ def op_fixture(
         opts["as_sql"] = as_sql
     if literal_binds:
         opts["literal_binds"] = literal_binds
-    if not sqla_14 and dialect == "mariadb":
-        ctx_dialect = _get_dialect("mysql")
-        ctx_dialect.server_version_info = (10, 4, 0, "MariaDB")
 
-    else:
-        ctx_dialect = _get_dialect(dialect)
+    ctx_dialect = _get_dialect(dialect)
     if native_boolean is not None:
         ctx_dialect.supports_native_boolean = native_boolean
         # this is new as of SQLAlchemy 1.2.7 and is used by SQL Server,
@@ -220,7 +210,6 @@ def op_fixture(
 
 
 class AlterColRoundTripFixture:
-
     # since these tests are about syntax, use more recent SQLAlchemy as some of
     # the type / server default compare logic might not work on older
     # SQLAlchemy versions as seems to be the case for SQLAlchemy 1.1 on Oracle
@@ -273,9 +262,11 @@ class AlterColRoundTripFixture:
                 "x",
                 column.name,
                 existing_type=column.type,
-                existing_server_default=column.server_default
-                if column.server_default is not None
-                else False,
+                existing_server_default=(
+                    column.server_default
+                    if column.server_default is not None
+                    else False
+                ),
                 existing_nullable=True if column.nullable else False,
                 # existing_comment=column.comment,
                 nullable=to_.get("nullable", None),
@@ -303,9 +294,13 @@ class AlterColRoundTripFixture:
             new_col["type"],
             new_col.get("default", None),
             compare.get("type", old_col["type"]),
-            compare["server_default"].text
-            if "server_default" in compare
-            else column.server_default.arg.text
-            if column.server_default is not None
-            else None,
+            (
+                compare["server_default"].text
+                if "server_default" in compare
+                else (
+                    column.server_default.arg.text
+                    if column.server_default is not None
+                    else None
+                )
+            ),
         )
